import 'package:crypto/crypto.dart';
import 'dart:convert';

import 'dart:typed_data';

import 'package:crypto_keys/crypto_keys.dart';

import "package:pointycastle_base/pointycastle.dart";
import "package:pointycastle_base/api.dart";
import "package:pointycastle_base/digests/sha1.dart";
import "package:pointycastle_base/macs/hmac.dart";
import "package:pointycastle_base/key_derivators/pbkdf2.dart";
import 'package:crosspki/crosspki.dart';

import 'model/model.dart';
import 'dart:math';

class CommonCrypto {
  static String calculateSha256(String value) {
    return base64.encode(sha256.convert(utf8.encode(value)).bytes);
  }
  static String calculateHmac(String value, String key) {
    var hmac = Hmac(sha256, base64.decode(value));
    var digest = hmac.convert(base64.decode(key));
    return base64.encode(digest.bytes);
  }
  static String calculateDoubleSha256(String value) {
    return base64
        .encode(sha256.convert(sha256.convert(utf8.encode(value)).bytes).bytes);
  }
  static String signChallengeHash(String challenge, String key) {
    var digest =
    new Hmac(sha256, base64.decode(challenge)).convert(base64.decode(key));
    return base64.encode(digest.bytes);
  }
  static KeyPair getPublicKeyByKeyPair(KeyPairInfoModel kp) {
    return getPublicKey(kp.publicKeyModulus ,kp.publicKeyExponent);
  }
  static KeyPair getPublicKey(String publicKeyModulus, String publicKeyExponent) {
    return new KeyPair.fromJwk({
      "kty": "RSA",
      "n": publicKeyModulus,
      "e": publicKeyExponent,
    });
  }
  static EncryptionResult rsaEncrypt(KeyPair publicKey, Uint8List content) {
    // See: https://pub.dev/packages/crypto_keys
    var encrypter = publicKey.publicKey.createEncrypter(algorithms.encryption.rsa.oaep);
    return encrypter.encrypt(content);
  }
  static String pbkdf2ByKeyPair(String password, KeyPairInfoModel kp) {
    return pbkdf2(password, kp.pinSalt, kp.iterationCount);
  }
  static String pbkdf2(String password, String pinSalt, int iterationCount) {
    var salt = base64Decode(pinSalt);
    KeyDerivator derivator = new PBKDF2KeyDerivator(new HMac(new SHA1Digest(), 64));
    Pbkdf2Parameters params = new Pbkdf2Parameters(salt, iterationCount, salt.length);
    derivator.init(params);
    var key = derivator.process(new Uint8List.fromList(password.codeUnits));
    return base64Encode(key);
  }
  static Future<Uint8List> signFromPkcs12ByKeyPair(String password, KeyPairInfoModel kp, Uint8List data) async {
    return signFromPkcs12(password, kp.pinSalt, kp.iterationCount, kp.pkcs12, data);
  }
  static Future<Uint8List> signFromPkcs12(String password, String pinSalt, int iterationCount, String pkcs12, Uint8List data) async {
    var keyB64 = CommonCrypto.pbkdf2(password, pinSalt, iterationCount);
    await CrossPki.importPkcs12(base64Decode(pkcs12), keyB64);
    var certs = await CrossPki.listCertificatesWithKey();
    var sig = await CrossPki.signData(certs[0].thumbprint, DigestAlgorithm.sha256, data);
    for(var i=0; i<certs.length; i++) {
      await CrossPki.removeCertificate(certs[i].thumbprint);
    }
    return sig;
  }
  static Future loadPkcs12(String password, String pinSalt, int iterationCount, String pkcs12) async {
    var keyB64 = CommonCrypto.pbkdf2(password, pinSalt, iterationCount);
    await CrossPki.importPkcs12(base64Decode(pkcs12), keyB64);
  }
  static Future<Uint8List> signFromCurrentPkcs12(Uint8List data) async {
    var certs = await CrossPki.listCertificatesWithKey();
    var sig = await CrossPki.signData(certs[0].thumbprint, DigestAlgorithm.sha256, data);
    return sig;
  }
  static Future clearPkcs12() async {
    var certs = await CrossPki.listCertificatesWithKey();
    for(var i=0; i<certs.length; i++) {
      await CrossPki.removeCertificate(certs[i].thumbprint);
    }
  }

  static var _chars = 'AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz1234567890';
  static Random _rnd = Random();
  static String getRandomString(int length) => String.fromCharCodes(Iterable.generate(length, (_) => _chars.codeUnitAt(_rnd.nextInt(_chars.length))));
}
